@@ -309,7 +309,7 @@ class WebsocketTransport:
 
     def flush(self) -> None:
         if self._buffer:
-            asyncio.ensure_future(self.send(self._buffer), loop=self.loop)
+            asyncio.ensure_future(self.send(self._buffer))
 
         self._buffer = b''
 
@@ -425,7 +425,7 @@ class XMPPOverWebsocketConnector(aioxmpp.connector.BaseConnector):
                       ) -> Tuple[WebsocketTransport,
                                  WebsocketXMLStream,
                                  aioxmpp.nonza.StreamFeatures]:
-        features_future = asyncio.Future(loop=loop)
+        features_future = asyncio.Future()
 
         stream = WebsocketXMLStream(
             to=domain,
@@ -463,10 +463,15 @@ class XMPPClient:
 
         self.xmpp_client = None
         self.stream = None
-        self._ping_task = None
-        self._task = None
         self.muc_room = None
 
+        self._ping_task = None
+        self._is_suspended = False
+        self._reconnect_recover_task = None
+        self._last_disconnected_at = None
+        self._last_known_party_id = None
+        self._task = None
+
     def jid(self, user_id: str) -> aioxmpp.JID:
         return aioxmpp.JID.fromstr('{}@{}'.format(
             user_id,
@@ -556,7 +561,7 @@ class XMPPClient:
 
             data = self.client.get_user(_id)
             if data is None:
-                data = await self.client.fetch_profile(_id, raw=True)
+                data = await self.client.fetch_user(_id, raw=True)
             else:
                 data = data.get_raw()
 
@@ -574,7 +579,7 @@ class XMPPClient:
             })
 
             try:
-                self.client._pending_friends.remove(f.id)
+                del self.client._pending_friends[f.id]
             except KeyError:
                 pass
 
@@ -583,7 +588,7 @@ class XMPPClient:
         elif _status == 'PENDING':
             data = self.client.get_user(_id)
             if data is None:
-                data = await self.client.fetch_profile(_id, raw=True)
+                data = await self.client.fetch_user(_id, raw=True)
             else:
                 data = data.get_raw()
 
@@ -612,22 +617,37 @@ class XMPPClient:
         if body['reason'] == 'ABORTED':
             pf = self.client.get_pending_friend(_id)
             self.client.store_user(pf.get_raw())
-            self.client._pending_friends.remove(pf.id)
+
+            try:
+                del self.client._pending_friends[pf.id]
+            except KeyError:
+                pass
+
             self.client.dispatch_event('friend_request_abort', pf)
         elif body['reason'] == 'REJECTED':
             pf = self.client.get_pending_friend(_id)
             self.client.store_user(pf.get_raw())
-            self.client._pending_friends.remove(pf.id)
+
+            try:
+                del self.client._pending_friends[pf.id]
+            except KeyError:
+                pass
+
             self.client.dispatch_event('friend_request_decline', pf)
         else:
             f = self.client.get_friend(_id)
             if f is not None:
                 self.client.store_user(f.get_raw())
-                self.client._friends.remove(f.id)
+
+                try:
+                    del self.client._friends[f.id]
+                except KeyError:
+                    pass
+
                 self.client.dispatch_event('friend_remove', f)
 
         try:
-            self.client._presences.remove(_id)
+            del self.client._presences[_id]
         except KeyError:
             pass
 
@@ -636,7 +656,7 @@ class XMPPClient:
         body = ctx.body
 
         account_id = body['payload']['accountId']
-        data = await self.client.fetch_profile(account_id, raw=True)
+        data = await self.client.fetch_user(account_id, raw=True)
         blocked_user = self.client.store_blocked_user(data)
         self.client.dispatch_event('user_block', blocked_user)
 
@@ -645,9 +665,14 @@ class XMPPClient:
         body = ctx.body
 
         account_id = body['payload']['accountId']
-        profile = await self.client.fetch_profile(account_id)
-        self.client._blocked_users.remove(profile.id)
-        self.client.dispatch_event('user_unblock', profile)
+        user = await self.client.fetch_user(account_id)
+
+        try:
+            del self.client._blocked_users[user.id]
+        except KeyError:
+            pass
+
+        self.client.dispatch_event('user_unblock', user)
 
     @dispatcher.event('com.epicgames.social.party.notification.v0.PING')
     async def event_ping_received(self, ctx: EventContext) -> None:
@@ -655,14 +680,17 @@ class XMPPClient:
         pinger = body['pinger_id']
         try:
             data = (await self.client.http.party_lookup_ping(pinger))[0]
-        except IndexError:
-            return
-        except HTTPException as exc:
-            m = 'errors.com.epicgames.social.party.ping_not_found'
-            if exc.message_code == m:
-                return
+        except (IndexError, HTTPException) as exc:
+            if isinstance(exc, HTTPException):
+                m = 'errors.com.epicgames.social.party.ping_not_found'
+                if exc.message_code != m:
+                    raise
 
-            raise
+            self.client.dispatch_event(
+                'invalid_party_invite',
+                self.client.get_friend(pinger)
+            )
+            return
 
         for inv in data['invites']:
             if inv['sent_by'] == pinger and inv['status'] == 'SENT':
@@ -716,7 +744,7 @@ class XMPPClient:
         if party.id != body.get('party_id'):
             return
 
-        member = party.members.get(user_id)
+        member = party.get_member(user_id)
         if member is None:
             member = (await party._update_members(
                 (body,),
@@ -728,10 +756,7 @@ class XMPPClient:
 
         yielding = party.me._default_config.yield_leadership
         if party.me and party.me.leader and not yielding:
-            fut = asyncio.ensure_future(
-                party.refresh_squad_assignments(),
-                loop=self.client.loop
-            )
+            fut = asyncio.ensure_future(party.refresh_squad_assignments())
 
         try:
             if member.id == self.client.user.id:
@@ -769,7 +794,7 @@ class XMPPClient:
         if party.id != body.get('party_id'):
             return
 
-        member = party.members.get(user_id)
+        member = party.get_member(user_id)
         if member is None:
             return
 
@@ -780,39 +805,9 @@ class XMPPClient:
 
         self.client.dispatch_event('party_member_leave', member)
 
-    @dispatcher.event('com.epicgames.social.party.notification.v0.MEMBER_EXPIRED')  # noqa
-    async def event_party_member_expired(self, ctx: EventContext) -> None:
-        body = ctx.body
-
-        user_id = body.get('account_id')
-        if user_id != self.client.user.id:
-            await self.client._join_party_lock.wait()
-
-        party = self.client.party
-
-        if party is None:
-            return
-
-        if party.id != body.get('party_id'):
-            return
-
-        member = party.members.get(user_id)
-        if member is None:
-            return
-
-        party._remove_member(member.id)
-
-        if party.me and party.me.leader and member.id != party.me.id:
-            await party.refresh_squad_assignments()
-
-        if member.id == self.client.user.id:
-            p = await self.client._create_party()
-            self.client.party = p
-
-        self.client.dispatch_event('party_member_expire', member)
-
     @dispatcher.event('com.epicgames.social.party.notification.v0.MEMBER_KICKED')  # noqa
     async def event_party_member_kicked(self, ctx: EventContext) -> None:
+        return
         body = ctx.body
 
         user_id = body.get('account_id')
@@ -827,7 +822,7 @@ class XMPPClient:
         if party.id != body.get('party_id'):
             return
 
-        member = party.members.get(user_id)
+        member = party.get_member(user_id)
         if member is None:
             return
 
@@ -847,6 +842,41 @@ class XMPPClient:
     @dispatcher.event('com.epicgames.social.party.notification.v0.MEMBER_DISCONNECTED')  # noqa
     async def event_party_member_disconnected(self, ctx: EventContext) -> None:
         body = ctx.body
+        user_id = body.get('account_id')
+
+        # Dont continue processing for old connections
+        data = await self.client.http.party_lookup(self.client.party.id)
+        for member_data in data['members']:
+            if member_data['account_id'] == user_id:
+                connections = member_data['connections']
+                if len(connections) == 1:
+                    break
+
+                for connection in connections:
+                    if 'disconnected_at' not in connection:
+                        return
+
+        if user_id != self.client.user.id:
+            await self.client._join_party_lock.wait()
+
+        party = self.client.party
+
+        if party is None:
+            return
+
+        if party.id != body.get('party_id'):
+            return
+
+        member = party.get_member(user_id)
+        if member is None:
+            return
+
+        member._update_connection(body.get('connection'))
+        self.client.dispatch_event('party_member_zombie', member)
+
+    @dispatcher.event('com.epicgames.social.party.notification.v0.MEMBER_EXPIRED')  # noqa
+    async def event_party_member_expired(self, ctx: EventContext) -> None:
+        body = ctx.body
 
         user_id = body.get('account_id')
         if user_id != self.client.user.id:
@@ -860,7 +890,7 @@ class XMPPClient:
         if party.id != body.get('party_id'):
             return
 
-        member = party.members.get(user_id)
+        member = party.get_member(user_id)
         if member is None:
             return
 
@@ -869,7 +899,34 @@ class XMPPClient:
         if party.me and party.me.leader and member.id != party.me.id:
             await party.refresh_squad_assignments()
 
-        self.client.dispatch_event('party_member_disconnect', member)
+        if member.id == self.client.user.id:
+            p = await self.client._create_party()
+            self.client.party = p
+
+        self.client.dispatch_event('party_member_expire', member)
+
+    @dispatcher.event('com.epicgames.social.party.notification.v0.MEMBER_CONNECTED')  # noqa
+    async def event_party_member_connected(self, ctx: EventContext) -> None:
+        body = ctx.body
+
+        user_id = body.get('account_id')
+        if user_id != self.client.user.id:
+            await self.client._join_party_lock.wait()
+
+        party = self.client.party
+
+        if party is None:
+            return
+
+        if party.id != body.get('party_id'):
+            return
+
+        member = party.get_member(user_id)
+        if member is None:
+            return
+
+        member._update_connection(body.get('connection'))
+        self.client.dispatch_event('party_member_reconnect', member)
 
     @dispatcher.event('com.epicgames.social.party.notification.v0.MEMBER_NEW_CAPTAIN')  # noqa
     async def event_party_new_captain(self, ctx: EventContext) -> None:
@@ -888,17 +945,12 @@ class XMPPClient:
         if party.id != body.get('party_id'):
             return
 
-        member = party.members.get(user_id)
+        member = party.get_member(user_id)
         if member is None:
             return
 
         old_leader = party.leader
-        for m in party.members.values():
-            m.update_role(None)
-
-        member.update_role('CAPTAIN')
-        if member.id == self.client.user.id:
-            self.client.party.me.update_role('CAPTAIN')
+        party._update_roles(member)
 
         party.update_presence()
         self.client.dispatch_event('party_member_promote', old_leader, member)
@@ -959,7 +1011,7 @@ class XMPPClient:
         if party.id != body.get('party_id'):
             return
 
-        member = party.members.get(user_id)
+        member = party.get_member(user_id)
         if member is None:
             def check(m):
                 return m.id == user_id
@@ -1030,7 +1082,7 @@ class XMPPClient:
                         try:
                             self.client.dispatch_event(
                                 'party_member_team_swap',
-                                *[party.members[k] for k in new_positions]
+                                *[party._members[k] for k in new_positions]
                             )
                         except KeyError:
                             pass
@@ -1081,9 +1133,14 @@ class XMPPClient:
 
         user = self.client.get_user(user_id)
         if user is None:
-            user = await self.client.fetch_profile(user_id)
+            user = await self.client.fetch_user(user_id)
 
         confirmation = PartyJoinConfirmation(self.client, party, user, body)
+
+        # Automatically confirm if event is received but no handler is found.
+        if not self.client._event_has_destination('party_member_confirm'):
+            return await confirmation.confirm()
+
         self.client.dispatch_event('party_member_confirm', confirmation)
 
     @dispatcher.event('com.epicgames.social.party.notification.v0.INVITE_DECLINED')  # noqa
@@ -1151,13 +1208,96 @@ class XMPPClient:
 
         if not is_available and friend.is_online():
             friend._update_last_logout(datetime.datetime.utcnow())
-            self.client._presences.remove(user_id, None)
+
+            try:
+                del self.client._presences[user_id]
+            except KeyError:
+                pass
+
         else:
-            self.client._presences.set(user_id, _pres)
+            self.client._presences[user_id] = _pres
 
         self.client.dispatch_event('friend_presence', before_pres, _pres)
 
+    def on_stream_established(self):
+        self.client.dispatch_event('xmpp_session_establish')
+
+        async def on_establish():
+            # Just incase the recover task hangs we don't want it
+            # running forever in the background until a new close is
+            # dispatched potentially fucking shit up big time.
+            task = self._reconnect_recover_task
+            if task is not None and not task.done():
+                try:
+                    await asyncio.wait_for(task, timeout=20)
+                except asyncio.TimeoutError:
+                    pass
+
+        async def party_reconnect():
+            now = datetime.datetime.utcnow()
+            secs = (now - self._last_disconnected_at).total_seconds()
+            if secs >= self.client.default_party_member_config.offline_ttl:
+                return await self.client._create_party()
+
+            data = await self.client.http.party_lookup_user(
+                self.client.user.id
+            )
+            if data['current']:
+                party_data = data['current'][0]
+                try:
+                    async with self.client._join_party_lock:
+                        await self.client._join_party(
+                            party_data,
+                            event='party_member_reconnect'
+                        )
+                except Exception:
+                    await self.client._create_party(acquire=True)
+                    raise
+            else:
+                await self.client._create_party()
+
+        if self._is_suspended:
+            self.client.dispatch_event('xmpp_session_reconnect')
+            self.client.loop.create_task(party_reconnect())
+
+        self._is_suspended = False
+
+        self.client.loop.create_task(on_establish())
+
+    def on_stream_suspended(self, reason):
+        jid = self.xmpp_client.local_jid
+        resource = jid.resource[:-32] + (uuid.uuid4().hex).upper()
+        self.xmpp_client._local_jid = jid.replace(resource=resource)
+
+        def on_events_recovered(*args):
+            self._reconnect_recover_task = None
+
+        self._reconnect_recover_task = task = self.client.loop.create_task(
+            self.client.recover_events(
+                refresh_caches=True,
+                wait_for_close=False
+            )
+        )
+        task.add_done_callback(on_events_recovered)
+
+        if self.client.party is not None:
+            self._last_known_party_id = self.client.party.id
+
+        self._last_disconnected_at = datetime.datetime.utcnow()
+        self._is_suspended = True
+        self.client.dispatch_event('xmpp_session_lost')
+
+    def on_stream_destroyed(self, reason=None):
+        if not self._is_suspended:
+            task = self._reconnect_recover_task
+            if task is not None and not task.cancelled():
+                task.cancel()
+
+        self.client.dispatch_event('xmpp_session_close')
+
     def setup_callbacks(self, messages: bool = True) -> None:
+        client = self.xmpp_client
+
         message_dispatcher = self.xmpp_client.summon(
             aioxmpp.dispatcher.SimpleMessageDispatcher
         )
@@ -1167,11 +1307,14 @@ class XMPPClient:
                 aioxmpp.MessageType.CHAT,
                 None,
                 lambda m: asyncio.ensure_future(
-                    self.process_chat_message(m),
-                    loop=self.client.loop
+                    self.process_chat_message(m)
                 ),
             )
 
+        client.on_stream_established.connect(self.on_stream_established)
+        client.on_stream_suspended.connect(self.on_stream_suspended)
+        client.on_stream_destroyed.connect(self.on_stream_destroyed)
+
     async def loop_ping(self) -> None:
         while True:
             await asyncio.sleep(60)
@@ -1185,8 +1328,8 @@ class XMPPClient:
     async def _run(self, future: asyncio.Future) -> None:
         async with self.xmpp_client.connected() as stream:
             self.stream = stream
-            stream.soft_timeout = datetime.timedelta(minutes=5)
-            stream.round_trip_time = datetime.timedelta(minutes=5)
+            stream.soft_timeout = datetime.timedelta(minutes=3)
+            stream.round_trip_time = datetime.timedelta(minutes=3)
             future.set_result(None)
             while True:
                 await asyncio.sleep(1)
@@ -1224,20 +1367,16 @@ class XMPPClient:
         self.setup_callbacks()
 
         future = self.client.loop.create_future()
-        self._task = asyncio.ensure_future(self._run(future),
-                                           loop=self.client.loop)
+        self._task = asyncio.ensure_future(self._run(future))
         await future
-        self._ping_task = asyncio.ensure_future(self.loop_ping(),
-                                                loop=self.client.loop)
 
-        self.client.dispatch_event('xmpp_session_establish')
+        self._ping_task = asyncio.ensure_future(self.loop_ping())
 
     async def close(self) -> None:
         log.debug('Attempting to close xmpp client')
         if self.xmpp_client.running:
             self.xmpp_client.stop()
 
-            # let the loop run one more time so the xmpp client can stop
             while self.xmpp_client.running:
                 await asyncio.sleep(0)
 
@@ -1251,7 +1390,6 @@ class XMPPClient:
         self.stream = None
         self.muc_service = None
         log.debug('Successfully closed xmpp client')
-        self.client.dispatch_event('xmpp_session_close')
 
         # let loop run one iteration for events to be dispatched
         await asyncio.sleep(0)
@@ -1267,13 +1405,13 @@ class XMPPClient:
         party = self.client.party
 
         if (user_id == self.client.user.id or member.nick is None
-                or user_id not in party.members):
+                or user_id not in party._members):
             return
 
         self.client.dispatch_event('party_message', PartyMessage(
             client=self.client,
             party=party,
-            author=party.members[member.direct_jid.localpart],
+            author=party._members[member.direct_jid.localpart],
             content=message.body.any()
         ))
 
@@ -1289,7 +1427,7 @@ class XMPPClient:
                      muc_reason: str,
                      **kwargs: Any) -> None:
         if muc_leave_mode is aioxmpp.muc.LeaveMode.BANNED:
-            mem = self.client.party.members[member.direct_jid.localpart]
+            mem = self.client.party._members[member.direct_jid.localpart]
             self.client.dispatch_event('party_member_chatban',
                                        mem,
                                        muc_reason)
@@ -1315,7 +1453,7 @@ class XMPPClient:
         room.on_leave.connect(self.muc_on_leave)
         self.muc_room = room
 
-        asyncio.ensure_future(fut, loop=self.client.loop)
+        asyncio.ensure_future(fut)
         await self.client.wait_for('muc_enter')
 
     async def leave_muc(self) -> None:
@@ -1357,24 +1495,24 @@ class XMPPClient:
         msg.body[None] = content
         await self.stream.send(msg)
 
-    def set_presence(self, status: Optional[Union[str, dict]] = None) -> None:
+    def set_presence(self, *,
+                     status: Optional[Union[str, dict]] = None,
+                     show: Optional[str]) -> None:
         if status is None:
             self.xmpp_client.presence = aioxmpp.PresenceState(available=True)
 
-        elif isinstance(status, dict):
-            self.xmpp_client.set_presence(
-                aioxmpp.PresenceState(available=True),
-                json.dumps(status),
-            )
-
-        else:
-            self.xmpp_client.set_presence(
-                aioxmpp.PresenceState(available=True),
-                json.dumps({'Status': status}),
-            )
+        _status = status if isinstance(status, dict) else {'Status': status}
+        self.xmpp_client.set_presence(
+            state=aioxmpp.PresenceState(
+                available=True,
+                show=show
+            ),
+            status=json.dumps(_status)
+        )
 
     async def send_presence(self, to: Optional[aioxmpp.JID] = None,
-                            status: Optional[Union[str, dict]] = None) -> None:
+                            status: Optional[Union[str, dict]] = None,
+                            show: Optional[str] = None) -> None:
         _status = {}
         if status is None:
             _status = None
@@ -1385,7 +1523,11 @@ class XMPPClient:
         else:
             raise TypeError('status must be None, str or dict')
 
-        pres = aioxmpp.Presence(type_=aioxmpp.PresenceType.AVAILABLE, to=to)
+        pres = aioxmpp.Presence(
+            type_=aioxmpp.PresenceType.AVAILABLE,
+            show=aioxmpp.PresenceShow(show),
+            to=to,
+        )
 
         if _status is not None:
             pres.status[None] = json.dumps(_status)
